#pragma once
#include <map>
#include <string>
#include <fstream>
#include <iostream>
#include <iterator>
#include <algorithm>

struct Date {
	int day;
	int month;
	int year;

	bool operator ==(const Date& to_compare) { return (day == to_compare.day) && (month == to_compare.month) && (year == to_compare.year); };
	bool operator <(const Date& to_compare) {
		if (year < to_compare.year) {
			return true;
		} else if (year == to_compare.year) {
			if (month < to_compare.month) {
				return true;
			} else if (month == to_compare.month) {
				if (day < to_compare.day) {
					return true;
				} else {
					return false;
				}
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

	Date operator +(const Date& to_summ) {
		day += to_summ.day;
		while (day >= 30) {
			day = day - 30;
			month++;
		}
		month += to_summ.month;
		while (month >= 12) {
			month = month - 12;
			year++;
		}
		year += to_summ.year;
		return *this;
	}

	Date operator +(const int& to_summ) {
		day += to_summ;
		while (day >= 30) {
			day = day - 30;
			month++;
		}
		while (month >= 12) {
			month = month - 12;
			year++;
		}
		return *this;
	}

	friend std::ostream& operator << (std::ostream& out, const Date& p_date) { out << p_date.day << "." << p_date.month << "." << p_date.year << std::endl; return out;};
	friend std::istream& operator >> (std::istream& in, Date& p_date) {
		std::cout << "day = ";
		in >> p_date.day;
		std::cout << "month = ";
		in >> p_date.month;
		std::cout << "year = ";
		in >> p_date.year;
		return in;
	}
};

struct Production {
	int storage_id;
	int product_code;
	char product_name[20];
	Date arrival_date;
	int shelf_life;
	int amount_of_products;
	double unit_price;

	bool operator ==(const Production& prod_to_comp) {
		return storage_id == prod_to_comp.storage_id &&
			product_code == prod_to_comp.product_code &&
			strcmp(product_name, prod_to_comp.product_name) &&
			arrival_date == prod_to_comp.arrival_date &&
			shelf_life == prod_to_comp.shelf_life &&
			amount_of_products == prod_to_comp.amount_of_products &&
			unit_price == prod_to_comp.unit_price;

	}

	friend std::ostream& operator <<(std::ostream& out, const Production& P) {
		out << "Storage id:  " << P.storage_id << std::endl
			<< "Product code:  " << P.product_code << std::endl
			<< "Product name:  " << P.product_name << std::endl
			<< "Arrival date:  " << P.arrival_date << std::endl
			<< "Shelf life :  " << P.shelf_life << std::endl
			<< "Amount of products:  " << P.amount_of_products << std::endl
			<< "Unit price:  " << P.unit_price << "c.u." << std::endl;
		return out;
	}

	friend std::istream& operator >>(std::istream& in, Production& P) {
		std::cout << "Storage id =  ";
		in >> P.storage_id;
		std::cout << "Product code =   ";
		in >> P.product_code;
		std::cout << "Product name =   ";
		in >> P.product_name;
		std::cout << "Arrival date:" << std::endl;
		in >> P.arrival_date;
		std::cout << "Shelf life =   ";
		in >> P.shelf_life;
		std::cout << "Amount of products =   ";
		in >> P.amount_of_products;
		std::cout << "Unit price =   ";
		in >> P.unit_price;
		return in;

	}
};

class Storage {
	std::map<size_t, Production> product;
	std::string bin_f_name;
	std::string txt_f_name;
	size_t f_amount;
	static void new_prod_insert(Production& tmp_prod);
	void load_in();

public:
	Storage(const std::string& bin_f, const std::string& txt_to_out);
	~Storage();

	void create_and_write();
	void printData()  noexcept;
	void print_product(const std::pair<size_t, Production>& prod);
	void update_file();

	void add_product();
	void delete_product(int prod_id);
	void change_product(int prod_id);

	void search_storage_id(int id) noexcept;
	void search_product_code(int code) noexcept;
	void search_arrival_date(const Date& arr_date) noexcept;
	void search_overdue_products(const Date& curr_date) noexcept;
	void search_not_overdue_products(const Date& curr_date) noexcept;

	void load_out_txt();
};

